@@ -27,7 +27,10 @@
  * https://www.rfc-editor.org/rfc/rfc8216.txt
  */
 
+#include "config_components.h"
+
 #include "libavformat/http.h"
+#include "libavutil/aes.h"
 #include "libavutil/avstring.h"
 #include "libavutil/avassert.h"
 #include "libavutil/intreadwrite.h"
@@ -36,6 +39,7 @@
 #include "libavutil/dict.h"
 #include "libavutil/time.h"
 #include "avformat.h"
+#include "demux.h"
 #include "internal.h"
 #include "avio_internal.h"
 #include "id3v2.h"
@@ -116,6 +120,8 @@ struct playlist {
     enum PlaylistType type;
     int64_t target_duration;
     int64_t start_seq_no;
+    int time_offset_flag;
+    int64_t start_time_offset;
     int n_segments;
     struct segment **segments;
     int needed;
@@ -207,6 +213,7 @@ typedef struct HLSContext {
     int64_t cur_seq_no;
     int m3u8_hold_counters;
     int live_start_index;
+    int prefer_x_start;
     int first_packet;
     int64_t first_timestamp;
     int64_t cur_timestamp;
@@ -886,6 +893,21 @@ static int parse_playlist(HLSContext *c, const char *url,
                 cur_init_section->key = NULL;
             }
 
+        } else if (av_strstart(line, "#EXT-X-START:", &ptr)) {
+            const char *time_offset_value = NULL;
+            ret = ensure_playlist(c, &pls, url);
+            if (ret < 0) {
+                goto fail;
+            }
+            if (av_strstart(ptr, "TIME-OFFSET=", &time_offset_value)) {
+                float offset = strtof(time_offset_value, NULL);
+                pls->start_time_offset = offset * AV_TIME_BASE;
+                pls->time_offset_flag = 1;
+            } else {
+                av_log(c->ctx, AV_LOG_WARNING, "#EXT-X-START value is"
+                                                "invalid, it will be ignored");
+                continue;
+            }
         } else if (av_strstart(line, "#EXT-X-ENDLIST", &ptr)) {
             if (pls)
                 pls->finished = 1;
@@ -1660,7 +1682,8 @@ static void add_metadata_from_renditions(AVFormatContext *s, struct playlist *pl
 /* if timestamp was in valid range: returns 1 and sets seq_no
  * if not: returns 0 and sets seq_no to closest segment */
 static int find_timestamp_in_playlist(HLSContext *c, struct playlist *pls,
-                                      int64_t timestamp, int64_t *seq_no)
+                                      int64_t timestamp, int64_t *seq_no,
+                                      int64_t *seg_start_ts)
 {
     int i;
     int64_t pos = c->first_timestamp == AV_NOPTS_VALUE ?
@@ -1675,6 +1698,9 @@ static int find_timestamp_in_playlist(HLSContext *c, struct playlist *pls,
         int64_t diff = pos + pls->segments[i]->duration - timestamp;
         if (diff > 0) {
             *seq_no = pls->start_seq_no + i;
+            if (seg_start_ts) {
+                *seg_start_ts = pos;
+            }
             return 1;
         }
         pos += pls->segments[i]->duration;
@@ -1698,7 +1724,7 @@ static int64_t select_cur_seq_no(HLSContext *c, struct playlist *pls)
      * playlist) and this is a complete file, find the matching segment
      * by counting durations. */
     if (pls->finished && c->cur_timestamp != AV_NOPTS_VALUE) {
-        find_timestamp_in_playlist(c, pls, c->cur_timestamp, &seq_no);
+        find_timestamp_in_playlist(c, pls, c->cur_timestamp, &seq_no, NULL);
         return seq_no;
     }
 
@@ -1715,9 +1741,45 @@ static int64_t select_cur_seq_no(HLSContext *c, struct playlist *pls)
         /* If this is a live stream, start live_start_index segments from the
          * start or end */
         if (c->live_start_index < 0)
-            return pls->start_seq_no + FFMAX(pls->n_segments + c->live_start_index, 0);
+            seq_no = pls->start_seq_no + FFMAX(pls->n_segments +
+                                            c->live_start_index, 0);
         else
-            return pls->start_seq_no + FFMIN(c->live_start_index, pls->n_segments - 1);
+            seq_no = pls->start_seq_no + FFMIN(c->live_start_index,
+                                            pls->n_segments - 1);
+
+        /* If #EXT-X-START in playlist, need to recalculate */
+        if (pls->time_offset_flag && c->prefer_x_start) {
+            int64_t start_timestamp;
+            int64_t playlist_duration = 0;
+            int64_t cur_timestamp = c->cur_timestamp == AV_NOPTS_VALUE ? 0 :
+                                    c->cur_timestamp;
+
+            for (int i = 0; i < pls->n_segments; i++)
+                playlist_duration += pls->segments[i]->duration;
+
+            /* If the absolute value of TIME-OFFSET exceeds
+             * the duration of the playlist, it indicates either the end of the
+             * playlist (if positive) or the beginning of the playlist (if
+             * negative). */
+            if (pls->start_time_offset >=0 &&
+                pls->start_time_offset > playlist_duration)
+                start_timestamp = cur_timestamp + playlist_duration;
+            else if (pls->start_time_offset >= 0 &&
+                        pls->start_time_offset <= playlist_duration)
+                start_timestamp = cur_timestamp + pls->start_time_offset;
+            else if (pls->start_time_offset < 0 &&
+                        pls->start_time_offset < -playlist_duration)
+                start_timestamp = cur_timestamp;
+            else if (pls->start_time_offset < 0 &&
+                        pls->start_time_offset > -playlist_duration)
+                start_timestamp = cur_timestamp + playlist_duration +
+                                    pls->start_time_offset;
+            else
+                start_timestamp = cur_timestamp;
+
+            find_timestamp_in_playlist(c, pls, start_timestamp, &seq_no, NULL);
+        }
+        return seq_no;
     }
 
     /* Otherwise just start on the first segment. */
@@ -2031,7 +2093,7 @@ static int hls_read_header(AVFormatContext *s)
         if (seg && seg->key_type == KEY_SAMPLE_AES && pls->is_id3_timestamped &&
             pls->audio_setup_info.codec_id != AV_CODEC_ID_NONE) {
             void *iter = NULL;
-            while ((in_fmt = (const AVInputFormat *)av_demuxer_iterate(&iter)))
+            while ((in_fmt = av_demuxer_iterate(&iter)))
                 if (in_fmt->raw_codec_id == pls->audio_setup_info.codec_id)
                     break;
         } else {
@@ -2058,7 +2120,7 @@ static int hls_read_header(AVFormatContext *s)
             if (strstr(in_fmt->name, "mov")) {
                 char key[33];
                 ff_data_to_hex(key, pls->key, sizeof(pls->key), 0);
-                av_dict_set(&options, "decryption_key", key, AV_OPT_FLAG_DECODING_PARAM);
+                av_dict_set(&options, "decryption_key", key, 0);
             } else if (!c->crypto_ctx.aes_ctx) {
                 c->crypto_ctx.aes_ctx = av_aes_alloc();
                 if (!c->crypto_ctx.aes_ctx) {
@@ -2369,7 +2431,7 @@ static int hls_read_seek(AVFormatContext *s, int stream_index,
     int i, j;
     int stream_subdemuxer_index;
     int64_t first_timestamp, seek_timestamp, duration;
-    int64_t seq_no;
+    int64_t seq_no, seg_start_ts;
 
     if ((flags & AVSEEK_FLAG_BYTE) || (c->ctx->ctx_flags & AVFMTCTX_UNSEEKABLE))
         return AVERROR(ENOSYS);
@@ -2379,8 +2441,7 @@ static int hls_read_seek(AVFormatContext *s, int stream_index,
 
     seek_timestamp = av_rescale_rnd(timestamp, AV_TIME_BASE,
                                     s->streams[stream_index]->time_base.den,
-                                    flags & AVSEEK_FLAG_BACKWARD ?
-                                    AV_ROUND_DOWN : AV_ROUND_UP);
+                                    AV_ROUND_DOWN);
 
     duration = s->duration == AV_NOPTS_VALUE ?
                0 : s->duration;
@@ -2401,9 +2462,16 @@ static int hls_read_seek(AVFormatContext *s, int stream_index,
     }
     /* check if the timestamp is valid for the playlist with the
      * specified stream index */
-    if (!seek_pls || !find_timestamp_in_playlist(c, seek_pls, seek_timestamp, &seq_no))
+    if (!seek_pls || !find_timestamp_in_playlist(c, seek_pls, seek_timestamp, &seq_no, &seg_start_ts))
         return AVERROR(EIO);
 
+    if (s->streams[stream_index]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO &&
+        flags & AVSEEK_FLAG_BACKWARD && !(flags & AVSEEK_FLAG_ANY)) {
+        /* Seeking to start of segment ensures we seek to a keyframe located
+         * before the given timestamp. */
+        seek_timestamp = seg_start_ts;
+    }
+
     /* set segment now so we do not need to search again below */
     seek_pls->cur_seq_no = seq_no;
     seek_pls->seek_stream_index = stream_subdemuxer_index;
@@ -2430,7 +2498,7 @@ static int hls_read_seek(AVFormatContext *s, int stream_index,
 
         if (pls != seek_pls) {
             /* set closest segment seq_no for playlists not handled above */
-            find_timestamp_in_playlist(c, pls, seek_timestamp, &pls->cur_seq_no);
+            find_timestamp_in_playlist(c, pls, seek_timestamp, &pls->cur_seq_no, NULL);
             /* seek the playlist to the given position without taking
              * keyframes into account since this playlist does not have the
              * specified stream where we should look for the keyframes */
@@ -2463,6 +2531,8 @@ static int hls_probe(const AVProbeData *p)
 static const AVOption hls_options[] = {
     {"live_start_index", "segment index to start live streams at (negative values are from the end)",
         OFFSET(live_start_index), AV_OPT_TYPE_INT, {.i64 = -3}, INT_MIN, INT_MAX, FLAGS},
+    {"prefer_x_start", "prefer to use #EXT-X-START if it's in playlist instead of live_start_index",
+        OFFSET(prefer_x_start), AV_OPT_TYPE_BOOL, { .i64 = 0 }, 0, 1, FLAGS},
     {"allowed_extensions", "List of file extensions that hls is allowed to access",
         OFFSET(allowed_extensions), AV_OPT_TYPE_STRING,
         {.str = "3gp,aac,avi,ac3,eac3,flac,mkv,m3u8,m4a,m4s,m4v,mpg,mov,mp2,mp3,mp4,mpeg,mpegts,ogg,ogv,oga,ts,vob,wav"},
@@ -2494,7 +2564,7 @@ const AVInputFormat ff_hls_demuxer = {
     .long_name      = NULL_IF_CONFIG_SMALL("Apple HTTP Live Streaming"),
     .priv_class     = &hls_class,
     .priv_data_size = sizeof(HLSContext),
-    .flags          = AVFMT_NOGENSEARCH | AVFMT_TS_DISCONT,
+    .flags          = AVFMT_NOGENSEARCH | AVFMT_TS_DISCONT | AVFMT_NO_BYTE_SEEK,
     .flags_internal = FF_FMT_INIT_CLEANUP,
     .read_probe     = hls_probe,
     .read_header    = hls_read_header,
